/* tserver.c - simple server for TCP/IP sockets */

/* Waits for a connection on port 1234. Once a connection has been
   established, copy data from the socket to stdout until the other 
   end closes the connection, and then wait for another connection to 
   the socket. */

#include <arpa/inet.h>
#include <netinet/in.h>
#include <stdio.h>
#include <signal.h>
#include <string.h>
#include <sys/socket.h>
#include <unistd.h>
#include <dirent.h>
#include <fcntl.h>
#include <sys/mman.h>



/* issue an error message via perror() and terminate the program */
void die(char * message) {
    perror(message);
    /* exit(1); */
}

int main(void) {
	struct sockaddr_in address;
	int sock, conn, i;
	char buf[2048];
	char class[4096];
	int classSize;
	int fd,classfd;
	int cont;
	int childpid=0;
	int f=0;
	int serve_applet,serve_stream;
	caddr_t dmap;
	FILE *fp=NULL;

	size_t addrLength = sizeof(struct sockaddr_in);
	signal(SIGPIPE,SIG_IGN);


	fd = open("/dev/display", O_RDWR);
	if (fd == -1)
		return 1;

	classfd = open("empegApplet.class", O_RDONLY);
	if (classfd == -1)
		return 2;

	classSize=read(classfd, class, sizeof(class));
	if ( classSize < 1 )
		return 3;

	dmap = mmap(0, 2048, PROT_READ | PROT_WRITE, MAP_SHARED, fd, 0);
	if (dmap == (caddr_t) -1)
		return 4;

	if ((sock = socket(PF_INET, SOCK_STREAM, 0)) < 0)
		return 5;

	/* Let the kernel reuse the socket address. This lets us run
	twice in a row, without waiting for the (ip, port) tuple
	to time out. */
	setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, &i, sizeof(i)); 

	address.sin_family = AF_INET;
	address.sin_port = htons(13470);
	memset(&address.sin_addr, 0, sizeof(address.sin_addr));

	if (bind(sock, (struct sockaddr *) &address, sizeof(address)))
		return 6;

	if (listen(sock, 5))
		return 7;

	f=fork();
	if ( f > 0 )
		exit(0);

	while ((conn = accept(sock, (struct sockaddr *) &address, 
                          &addrLength)) >= 0) {
		childpid=fork();
		if ( childpid > 0 ) {
			if ( (fp = fdopen( conn, "rw" ) ) == NULL ) {
				fprintf( stderr,"Cannot open stream\n");
				return 8;
			}
			while (strlen(fgets(buf, sizeof(buf),fp)) > 2 ) {
				if ( strstr(buf,"GET /empegApplet.class")!=NULL)
					serve_applet=1;
				if ( strstr(buf,"GET /empeg_stream") != NULL )
					serve_stream=1;
			}
			if ( serve_stream ) {
				cont=1;
				while ( cont ) {
					memcpy(&buf,dmap,2048);
					if( write(conn,buf,2048) != 2048)
						cont=0;
					usleep(50000);
				}
			} else if ( serve_applet ) {
				fputs("HTTP/1.0 200 OK\n",fp);
				fputs(
			"Server: EmpegCar DisplayServer/1.0 (Linux)\n",fp);
				fputs("Accept-Ranges: bytes\n",fp);
				fprintf(fp,"Content-Length: %d",classSize);
				fputs("Connection: close\n", fp);
				fputs(
			"Content-Type: application/octet-stream\n\n", fp);
				write(conn, class, classSize);
			}
		shutdown(conn,2);	
		close(conn);
		exit(0);
		}
	}

	if (conn < 0) 
		return 8;
	close(sock);
	if (munmap(dmap, 2048) == -1 || close(fd) == -1)
		return 3;
	return 0;
}


